Package: BufferedMatrix
Version: 1.76.0
Title: A matrix data storage object held in temporary files
Author: Ben Bolstad <bmb@bmbolstad.com>
Maintainer: Ben Bolstad <bmb@bmbolstad.com>
Depends: R (>= 2.6.0), methods
Description: A tabular style data object where most data is stored outside main memory. A buffer is used to speed up access to data.
License: LGPL (>= 2)
URL: https://github.com/bmbolstad/BufferedMatrix
Collate: allGenerics.R BufferedMatrix.R as.BufferedMatrix.R
        createBufferedMatrix.R
LazyLoad: yes
biocViews: Infrastructure
git_url: https://git.bioconductor.org/packages/BufferedMatrix
git_branch: RELEASE_3_23
git_last_commit: 9d72964
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 20:59:10 UTC; biocbuild
