setGeneric("buffer.dim", function(x) standardGeneric("buffer.dim"))
setGeneric("is.ColMode", function(x) standardGeneric("is.ColMode"))
setGeneric("is.RowMode", function(x) standardGeneric("is.RowMode"))
setGeneric("RowMode", function(x) standardGeneric("RowMode"))
setGeneric("ColMode", function(x) standardGeneric("ColMode"))
setGeneric("set.buffer.dim", function(x,rows,cols) standardGeneric("set.buffer.dim"))
setGeneric("prefix", function(x) standardGeneric("prefix"))
setGeneric("directory", function(x) standardGeneric("directory"))
setGeneric("filenames",function(x) standardGeneric("filenames"))
setGeneric("duplicate", function(x,...) standardGeneric("duplicate"))
setGeneric("ewApply", function(x,...) standardGeneric("ewApply"))
setGeneric("pow", function(x,...) standardGeneric("pow"))
setGeneric("Max", function(x,...) standardGeneric("Max"))
setGeneric("Min", function(x,...) standardGeneric("Min"))
setGeneric("Sum", function(x,...) standardGeneric("Sum"))
setGeneric("Var", function(x,...) standardGeneric("Var"))
setGeneric("Sd", function(x,...) standardGeneric("Sd"))
setGeneric("rowMeans", function(x,na.rm = FALSE, dims = 1L,...) standardGeneric("rowMeans"))
setGeneric("rowSums", function(x,na.rm = FALSE, dims = 1L,...) standardGeneric("rowSums"))
setGeneric("rowVars", function(x,na.rm = FALSE, dims = 1) standardGeneric("rowVars"))
setGeneric("rowSd", function(x,na.rm = FALSE, dims = 1) standardGeneric("rowSd"))
setGeneric("rowMax", function(x,na.rm = FALSE, dims = 1) standardGeneric("rowMax"))
setGeneric("rowMin", function(x,na.rm = FALSE, dims = 1) standardGeneric("rowMin"))
setGeneric("rowMedians", function(x,na.rm = FALSE) standardGeneric("rowMedians"))
setGeneric("colMeans", function(x,na.rm = FALSE, dims = 1L,...) standardGeneric("colMeans"))
setGeneric("colSums", function(x,na.rm = FALSE, dims = 1L,...) standardGeneric("colSums"))
setGeneric("colVars", function(x,na.rm = FALSE, dims = 1) standardGeneric("colVars"))
setGeneric("colSd", function(x,na.rm = FALSE, dims = 1) standardGeneric("colSd"))
setGeneric("colMax", function(x,na.rm = FALSE, dims = 1) standardGeneric("colMax"))
setGeneric("colMin", function(x,na.rm = FALSE, dims = 1) standardGeneric("colMin"))
setGeneric("colMedians", function(x,na.rm = FALSE) standardGeneric("colMedians"))
setGeneric("colRanges", function(x,na.rm = FALSE) standardGeneric("colRanges"))
setGeneric("colApply", function(x,...) standardGeneric("colApply"))
setGeneric("rowApply", function(x,...) standardGeneric("rowApply"))
setGeneric("subBufferedMatrix", function(x,...) standardGeneric("subBufferedMatrix"))
setGeneric("rownames", function(x,do.NULL=TRUE,prefix="row") standardGeneric("rownames"))
setGeneric("colnames", function(x,do.NULL=TRUE,prefix="col") standardGeneric("colnames"))
setGeneric("colnames<-", function(x,value) standardGeneric("colnames<-"))
setGeneric("rownames<-", function(x,value) standardGeneric("rownames<-"))
setGeneric("ReadOnlyMode", function(x) standardGeneric("ReadOnlyMode"))
setGeneric("is.ReadOnlyMode", function(x) standardGeneric("is.ReadOnlyMode"))
setGeneric("memory.usage",function(x) standardGeneric("memory.usage"))
setGeneric("disk.usage",function(x) standardGeneric("disk.usage"))
setGeneric("AddColumn",function(x) standardGeneric("AddColumn"))
setGeneric("MoveStorageDirectory",function(x,new.directory,...) standardGeneric("MoveStorageDirectory"))


